package com.example.android.constantesvitales;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.EditText;
import android.widget.TextView;

public class ActividadInicio extends AppCompatActivity {

    EditText nombreEditText;
    EditText edadEditText;
    EditText alturaEditText;
    EditText pesoEditText;
    TextView continuar;

    String nombre;
    int edad;
    int altura;
    int peso;

    SharedPreferences preferencias;


    /*ON CREATE*/

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.actividad_inicio);

        inicializarViews();
    }


    /* Inicializa todas las vistas y listeners */

    private void inicializarViews(){
        nombreEditText = (EditText) findViewById(R.id.nombre);
        edadEditText = (EditText) findViewById(R.id.edad);
        alturaEditText = (EditText) findViewById(R.id.altura);
        pesoEditText = (EditText) findViewById(R.id.peso);
        continuar = (TextView) findViewById(R.id.continuar);

        /*Cargamos los datos de la sesión anterior para que aparezcan ya en las editText views*/

        cargaDatos();

        /*Creamos un listener para pasar a la siguiente actividad y enviar a esta los datos del
        paciente recogidos*/

        continuar.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                try{
                    Paciente paciente = devuelveDatos();
                    guardaDatos(paciente);
                    Intent intent = new Intent(ActividadInicio.this,
                            com.example.android.constantesvitales.ActividadConstantes.class);
                    intent.putExtra("paciente", paciente);
                    startActivity(intent);
                }
                catch (Error e){
                    Log.e("Conectar", "Error al inicializar el perfil del paciente. " +
                            "Introduzca los datos correctamente", e);
                }
            }
        });
    }



    /*Devuelve una clase Paciente con los datos que el usuario ha introducido en los campos de
    texto*/

    private Paciente devuelveDatos() {

        /*En caso de que el usuario no introduzca alguno de los valores se introducen valores por
        defecto*/

        if (nombreEditText.getText().toString().matches(""))
            nombre = getString(R.string.sin_nombre);
        else
            nombre = nombreEditText.getText().toString();

        if (edadEditText.getText().toString().matches(""))
            edad = -1;
        else
            edad = Integer.parseInt(edadEditText.getText().toString());

        if (alturaEditText.getText().toString().matches(""))
            altura = -1;
        else
            altura = Integer.parseInt(alturaEditText.getText().toString());

        if (pesoEditText.getText().toString().matches(""))
            peso = -1;
        else
            peso = Integer.parseInt(pesoEditText.getText().toString());

        return new Paciente(nombre, edad, altura, peso);
    }



    /*Guardamos los datos introducidos para la siguiente sesión*/

    private void guardaDatos(Paciente paciente) {
        preferencias = getSharedPreferences("preferencias", Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = preferencias.edit();
        editor.putString("KEY_NOMBRE", paciente.getNombre());
        editor.putInt("KEY_EDAD", paciente.getEdad());
        editor.putInt("KEY_ALTURA", paciente.getAltura());
        editor.putInt("KEY_PESO", paciente.getPeso());
        editor.apply();
    }


    /*Cargamos las preferencias guardadas, y en caso de no encontrarse se introducen valores por
    defecto*/

    private void cargaDatos() {

        preferencias = getSharedPreferences("preferencias", Context.MODE_PRIVATE);
        nombre = preferencias.getString("KEY_NOMBRE", getString(R.string.sin_nombre));
        edad = preferencias.getInt("KEY_EDAD",-1);
        altura = preferencias.getInt("KEY_ALTURA",-1);
        peso = preferencias.getInt("KEY_PESO",-1);

        if(!(nombre.equals(getString(R.string.sin_nombre)))){
            nombreEditText.setText(nombre);
        }
        if(edad!=-1){
            edadEditText.setText(Integer.toString(edad));
        }
        if(altura!=-1){
            alturaEditText.setText(Integer.toString(altura));
        }
        if(peso!=-1){
            pesoEditText.setText(Integer.toString(peso));
        }
    }
}


