#include <ArduinoJson.h>

#define HC06 Serial3

#define B_RATE_RAPIDO 115200
#define B_RATE_LENTO 9600

#define SENSIBILIDAD_ALTA 2400
#define SENSIBILIDAD_BAJA 400

#define SIZE_VECTOR_MEDIA 10
#define SIZE_VECTOR_PENDIENTE 10

#define LED_IR 23
#define LED_R 24

#define Vo0 A0

bool pulsoAnterior;
bool pulso = false;
int ppm;
int oxigeno;
int duracion_pulso = 0;
int vectorPulsaciones[SIZE_VECTOR_MEDIA] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
unsigned long t_ciclo, t_muestreo, comienzo_pulso;
float V_IR, V_R, pendiente, limiteDeteccion, R;
float minimo_pendiente = 0;
float vector[SIZE_VECTOR_PENDIENTE] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
float extremos_IR[2] = {2400, 0};
float extremos_R[2] = {2400, 0};
float vectorR[SIZE_VECTOR_MEDIA] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};

//Declaramos el mensaje con formato json

StaticJsonDocument<200> mensaje;
JsonObject datos = mensaje.createNestedObject("datos");


/////////////////////////////////////////////////////////////////////////////
void setup()
{
  while (!Serial) continue;       //Esperamos a que se inicialice el puerto serie
  while (!HC06) continue;         //Esperamos a que se inicialice el puerto serie3 del bluetooth

  delay(1000);
  Serial.begin(B_RATE_LENTO);
  HC06.begin(B_RATE_RAPIDO);

  pinMode(LED_IR, OUTPUT);
  pinMode(LED_R, OUTPUT);

  pinMode(Vo0, INPUT);

  analogReference(AR_DEFAULT);    //Referencia de AnalogRead a 3.3V
  analogWriteResolution(10);      //10 bits (0-1023) para valores de tensión de salida
  analogReadResolution(12);       //12 bits (0-4095) para lectura de tensión

  digitalWrite(LED_IR, LOW);
  digitalWrite(LED_R, LOW);


  //Inicializamos el mensaje con formato json

  datos["ppg"] = 0;
  datos["oxigeno"] = 0;
  datos["pulso"] = 0;
}


/////////////////////////////////////////////////////////////////////////////
void loop()
{
  if (Check())
  {
    do
    {
      long unsigned ciclo = millis();
      while (millis() - ciclo < 1000)
      {
        t_muestreo=millis();
        Ciclo();
      }
      limiteDeteccion = minimo_pendiente * 1 / 3;
      minimo_pendiente = 0;
      enviaBluetooth();
    }
    while ((V_R < SENSIBILIDAD_ALTA) && (V_IR < SENSIBILIDAD_ALTA) && (V_R > SENSIBILIDAD_BAJA) && (V_IR > SENSIBILIDAD_BAJA));     //Hasta que alguna de las lecturas se salga del umbral de sensibilidad

    digitalWrite(LED_R, LOW);                   //Apagamos los dos leds
    digitalWrite(LED_IR, LOW);
  }
  else
  {
    long unsigned t = millis();

    for (int i = 0; i < SIZE_VECTOR_MEDIA; i++) //Reiniciamos los vectores
    {
      vectorPulsaciones[i] = 0;
      vectorR[i] = 0;
    }

    datos["ppg"] = 0;
    datos["oxigeno"] = 0;
    datos["pulso"] = 0;

    //enviaBluetooth();             //Enviamos un mensaje nulo para indicar que no estamos midiendo correctamente

    while (millis() - t < 500) continue;
  }
}


/////////////////////////////////////////////////////////////////////////////
void Pulso_LED(int led)
{
  unsigned long t = micros();
  float lectura;

  digitalWrite(led, HIGH);

  while ((micros() - t) < 300)
  {
    lectura = Lectura_Vo0();
  }

  digitalWrite(led, LOW);

  if (led == LED_IR)
    V_IR = lectura;
  else
    V_R = lectura;
}


//////////////////////////////////////////////////////////////////////////////
void Ciclo()
{
  t_ciclo = micros();

  Pulso_LED(LED_IR);
  while ((micros() - t_ciclo) < 500) continue;             //Semiperiodo del apagado de los leds

  Pulso_LED(LED_R);
  while ((micros() - t_ciclo) < 1000) continue;           //Semiperiodo del apagado de los leds

  pendiente = ActualizaVectorSenal(V_IR);
  minimo_pendiente = min(pendiente, minimo_pendiente);
  extremos_IR[0] = min(extremos_IR[0], V_IR);
  extremos_IR[1] = max(extremos_IR[1], V_IR);
  extremos_R[0] = min(extremos_R[0], V_R);
  extremos_R[1] = max(extremos_R[1], V_R);
  DetectaPulso();
  Envia_Datos();
}


//////////////////////////////////////////////////////////////////////////////
bool Check()
{
  unsigned long t = millis();
  float lectura_R, lectura_IR;

  lectura_R = DetectarDedo(LED_R);
  lectura_IR = DetectarDedo(LED_IR);

  if ((lectura_R < SENSIBILIDAD_ALTA) && (lectura_IR < SENSIBILIDAD_ALTA) &&
      (lectura_R > SENSIBILIDAD_BAJA) && (lectura_IR > SENSIBILIDAD_BAJA))          //Si estamos dentro del umbral de sensibilidad quiere decir que el sensor esta correctamente colocado en el dedo
  {
    return (true);
  }
  else
  {
    return (false);
  }
}


//////////////////////////////////////////////////////////////////////////////
float DetectarDedo(int led)
{
  unsigned long t = millis();
  float lectura = 0;
  int i = 1;
  digitalWrite(led, HIGH);
  while ((millis() - t) < 10)
  {
    lectura = (Lectura_Vo0() + (i - 1) * lectura) / i;                //Calculamos el valor medio de la lecturas
    i++;
  }
  digitalWrite(led, LOW);
  return (lectura);
}


//////////////////////////////////////////////////////////////////////////////
void Envia_Datos()
{
  Serial.print(V_IR);                         //V_IR
  Serial.print(",");
  Serial.print(V_R);                          //V_R
  Serial.print(",");
  Serial.print(pendiente);                    //Pendiente
  Serial.print(",");
  Serial.print(limiteDeteccion);              //Limite de deteccion
  Serial.print(",");
  //Serial.print(duracion_pulso);               //Duracion de pulso
  //Serial.print(",");
  //Serial.print(extremos_IR[0]);               //Mínimo de la señal IR
  //Serial.print(",");
  //Serial.print(extremos_IR[1]);               //Máximo de la señal IR
  //Serial.print(",");
  //Serial.print(extremos_R[0]);                //Mínimo de la señal R
  //Serial.print(",");
  //Serial.println(extremos_R[1]);              //Máximo de la señal IR
  //Serial.print(",");
  //Serial.print(ppm);                          //Pulsaciones por minuto
  //Serial.print(",");
  //Serial.print(R);                            //Relación R/IR
  //Serial.print(",");
  //Serial.print(oxigeno);                      //Saturación de oxígeno
  //Serial.print(",");
  if (pendiente < limiteDeteccion)            //Pulso
    Serial.println(2000);
  else
    Serial.println(0);

  //serializeJson(mensaje,Serial);
  //Serial.println();
  //serializeJson(mensaje, HC06);

}


//////////////////////////////////////////////////////////////////////////////
float ActualizaVectorSenal(float nuevo_valor)
{
  int i;
  float aux;
    for (i = (SIZE_VECTOR_PENDIENTE - 1); i >= 0; i--)
    {
      aux = vector[i];
      vector[i] = nuevo_valor;
      nuevo_valor = aux;
    }
  return ((vector[SIZE_VECTOR_PENDIENTE - 1] - vector[0]) * 10);
}


//////////////////////////////////////////////////////////////////////////////
void DetectaPulso()
{
  pulsoAnterior = pulso;

  if (pendiente < limiteDeteccion)
    pulso = true;
  else
    pulso = false;

  if ((!pulsoAnterior) && pulso)                            //Si detectamos pulso
  {
    duracion_pulso = millis() - comienzo_pulso;
    comienzo_pulso = millis();

    if ((duracion_pulso > 250) && (duracion_pulso < 1500))           //Si la variación no es fruto de ruidos
    {
      ActualizaVectorPulso(duracion_pulso);
      ActualizaVectorR(CalculaR());
      ppm = CalculaPPM();
      R = CalculaMediaR();
      oxigeno = CalculaSpO2(R);

      if (vectorPulsaciones[0] == 0 || oxigeno > 100 || oxigeno < 90)
      {
        datos["ppg"] = V_IR;
        datos["oxigeno"] = 0;
        datos["pulso"] = 0;
      }
      else
      {
        datos["ppg"] = V_IR;
        datos["oxigeno"] = oxigeno;
        datos["pulso"] = ppm;
      }
    }
    else if (duracion_pulso > 1500)
    {
      for (int i = 0; i < SIZE_VECTOR_MEDIA; i++) //Reiniciamos los vectores
      {
        vectorPulsaciones[i] = 0;
        vectorR[i] = 0;
      }
    }
  }
}


//////////////////////////////////////////////////////////////////////////////
void ActualizaVectorPulso(int nuevo_valor)
{
  int i;
  int aux;
  for (i = (SIZE_VECTOR_MEDIA - 1); i >= 0; i--)
  {
    aux = vectorPulsaciones[i];
    vectorPulsaciones[i] = nuevo_valor;
    nuevo_valor = aux;
  }
}


//////////////////////////////////////////////////////////////////////////////
void ActualizaVectorR(float nuevo_valor)
{
  int i;
  float aux;
  for (i = (SIZE_VECTOR_MEDIA - 1); i >= 0; i--)
  {
    aux = vectorR[i];
    vectorR[i] = nuevo_valor;
    nuevo_valor = aux;
  }
}


//////////////////////////////////////////////////////////////////////////////
int CalculaPPM()
{
  int i;
  float pulsos[SIZE_VECTOR_MEDIA];
  float suma_pulsos = 0;

  for (i = 0; i < SIZE_VECTOR_MEDIA; i++)
  {
    pulsos[i] = 60000 / vectorPulsaciones[i];
    suma_pulsos += pulsos[i];
  }

  return (int)((suma_pulsos / SIZE_VECTOR_MEDIA) + 0.5);   //Devolvemos el promedio de los tres pulsos, redondeando al entero más próximo
}


//////////////////////////////////////////////////////////////////////////////
float CalculaR()
{
  float r = (((extremos_R[1] - extremos_R[0]) * extremos_IR[0]) / ((extremos_IR[1] - extremos_IR[0]) * extremos_R[0]));
  extremos_IR[0] = 2400;
  extremos_IR[1] = 0;
  extremos_R[0] = 2400;
  extremos_R[1] = 0;

  return r;
}


//////////////////////////////////////////////////////////////////////////////
float CalculaMediaR()
{
  int i;
  float suma_R;

  for (i = 0; i < SIZE_VECTOR_MEDIA; i++)
  {
    suma_R += vectorR[i];
  }

  return (suma_R / SIZE_VECTOR_MEDIA);
}


//////////////////////////////////////////////////////////////////////////////
int CalculaSpO2(float r)
{
  return (int)(-8.1753 * r + 101.61 + 0.5);  //rednodeamos al entero más cercano
}


//////////////////////////////////////////////////////////////////////////////
float Lectura_Vo0()
{
  return (analogRead(Vo0) * 3.3 * 1000 / 4095);
}


//////////////////////////////////////////////////////////////////////////////
void enviaBluetooth()
{
  if(HC06.available())
  {
  serializeJson(mensaje, HC06);
  HC06.println();
  }
}


//////////////////////////////////////////////////////////////////////////////
void actualizaMensaje(float senal, float r, int pulsaciones)
{
  datos["ppg"] = senal;
  datos["oxigeno"] = r;
  datos["pulso"] = pulsaciones;
}
